/*
 Copyright (C) 2002 Anthony Van Groningen
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

//
//  JackMetroView.mm
//  jack_metro
//
//  Created by Christian Schoenebeck on 1/5/13.
//  Copyright (c) 2013 Crudebyte. All rights reserved.
//

#import "JackMetroView.h"
#import <jack/types.h>
#import <jack/jack.h>
#import <jack/custom.h>

/*
 This simple client from the standard JACK examples, implement a one audio output "bip" like test aplication.
 */

typedef jack_default_audio_sample_t sample_t;

// Globals
const double PI = 3.14;
jack_client_t *client;
jack_port_t *output_port;
unsigned long sr;
int freq = 880;
int bpm = 120;
jack_nframes_t tone_length, wave_length;
sample_t *wave;
long offset = 0;
int transport_aware = 0;
jack_transport_state_t transport_state;

// Audio process code
static void
process_silence (jack_nframes_t nframes)
{
	sample_t *buffer = (sample_t *) jack_port_get_buffer (output_port, nframes);
	memset (buffer, 0, sizeof (jack_default_audio_sample_t) * nframes);
}

static void
process_audio (jack_nframes_t nframes)
{
	sample_t *buffer = (sample_t *) jack_port_get_buffer (output_port, nframes);
	jack_nframes_t frames_left = nframes;

	while (wave_length - offset < frames_left) {
		memcpy (buffer + (nframes - frames_left), wave + offset, sizeof (sample_t) * (wave_length - offset));
		frames_left -= wave_length - offset;
		offset = 0;
	}
	if (frames_left > 0) {
		memcpy (buffer + (nframes - frames_left), wave + offset, sizeof (sample_t) * frames_left);
		offset += frames_left;
	}
}

static int
process (jack_nframes_t nframes, void *arg)
{
	if (transport_aware) {
		jack_position_t pos;

		if (jack_transport_query (client, &pos)
		    != JackTransportRolling) {

			process_silence (nframes);
			return 0;
		}
		offset = pos.frame % wave_length;
	}
	process_audio (nframes);
	return 0;
}

// Callback called when the server is closed
static void shut_down(void* arg)
{
    fprintf (stderr, "JACK server has been closed\n");
    exit(1);
}

static void info_shut_down(jack_status_t code, const char* reason, void* arg)
{
    fprintf (stderr, "%s\n", reason);
    exit(1);
}

// Callback called if the server buffer size is changed
static int buffer_size_callback(jack_nframes_t nframes, void *arg)
{
    fprintf (stdout,"buffer_size %d\n", nframes);
    return 0;
}

@implementation JackMetroView

- (int)runMetro
{
    sample_t scale;
	int i, attack_length, decay_length;
	double *amp;
	double max_amp = 0.5;
	int attack_percent = 1, decay_percent = 10, dur_arg = 100;
	const char *bpm_string = "bpm";
	jack_status_t status;
    int num_output = 1;
    char** physicalInPorts;
    
    // Open the client, fails if the server if not running
  	if ((client = jack_client_open ("metro", JackNullOption, &status)) == 0) {
        fprintf (stderr, "JACK server not running?\n");
		return 1;
	}
    // Setup callbacks
    jack_set_process_callback (client, process, 0);
    jack_on_shutdown(client, shut_down, 0);
    jack_on_info_shutdown(client, info_shut_down, 0);
    jack_set_buffer_size_callback(client, buffer_size_callback, 0);
    
    // Register a single audio port
	output_port = jack_port_register (client, bpm_string, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
  	sr = jack_get_sample_rate (client);
    
    int buffer_size = jack_get_buffer_size(client);
    fprintf (stdout,"buffer_size %d\n", buffer_size);
  
	// setup wave table parameters
	wave_length = 60 * sr / bpm;
	tone_length = sr * dur_arg / 1000;
	attack_length = tone_length * attack_percent / 100;
	decay_length = tone_length * decay_percent / 100;
	scale = 2 * PI * freq / sr;

	if (tone_length >= wave_length) {
		fprintf (stderr, "invalid duration (tone length = %u, wave length = %u\n", tone_length, wave_length);
		return -1;
	}
	if (attack_length + decay_length > (int)tone_length) {
		fprintf (stderr, "invalid attack/decay\n");
		return -1;
	}

	// Build the wave table 
	wave = (sample_t *) malloc (wave_length * sizeof(sample_t));
	amp = (double *) malloc (tone_length * sizeof(double));

	for (i = 0; i < attack_length; i++) {
		amp[i] = max_amp * i / ((double) attack_length);
	}
	for (i = attack_length; i < (int)tone_length - decay_length; i++) {
		amp[i] = max_amp;
	}
	for (i = (int)tone_length - decay_length; i < (int)tone_length; i++) {
		amp[i] = - max_amp * (i - (double) tone_length) / ((double) decay_length);
	}
	for (i = 0; i < (int)tone_length; i++) {
		wave[i] = amp[i] * sin (scale * i);
	}
	for (i = tone_length; i < (int)wave_length; i++) {
		wave[i] = 0;
	}

    // Activate the client
	if (jack_activate (client)) {
		fprintf (stderr, "cannot activate client\n");
		goto error;
	}
    
    // send the metronome app icon to JACK for being shown for this JACK client
    [self publishAppIcon];
    
    // Connect to "system" outputs
    physicalInPorts = (char**)jack_get_ports(client, NULL, NULL, JackPortIsPhysical|JackPortIsInput);
    if (physicalInPorts != NULL) {
        for (int i = 0; i < num_output && physicalInPorts[i]; i++) {
            jack_connect(client, jack_port_name(output_port), physicalInPorts[i]);
        }
        jack_free(physicalInPorts);
    }
    
    return 0;

error:
    free(amp);
    free(wave);    
    return -1;
}

- (void)quitMetro
{
    // Close client (this will remove it from the server)
    jack_client_close(client);
}

- (void)publishAppIcon
{
    NSString* iconFile = [[NSBundle mainBundle] pathForResource:@"Icon-Metro136" ofType:@"png"];
    NSFileHandle* fileHandle = [NSFileHandle fileHandleForReadingAtPath:iconFile];
    NSData* data = [fileHandle readDataToEndOfFile];
    const void* rawData = [data bytes];
    const size_t size   = [data length];
    NSLog(@"publishAppIcon rawDataSize = %ld", size);
    jack_custom_publish_data(client, "icon.png", rawData, size);
    [fileHandle closeFile];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

@end
