/*
  Copyright (c) 2013 Crudebyte, http://www.crudebyte.com

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _JACK_CUSTOM_H
#define _JACK_CUSTOM_H

#ifdef __cplusplus
extern "C" {
#endif

#include <jack/weakmacros.h>
#include <jack/types.h>
#include <stdlib.h>

/**
 * @file custom.h
 * @brief Sharing client specific, custom data among all clients
 *
 * Read @ref CustomAPI for more details about the purpose of this API part.
 */

/**
 * @defgroup CustomAPI Sharing client specific, custom data among all clients
 *
 * Purpose of this API is to allow arbitrary data of any type to be shared
 * between JACK clients. Data which would be out of the scope of the JACK
 * system itself. It is especially handy on sandboxed environments like iOS,
 * where all apps have their own, private and isolated file system, which
 * prevents data to be shared easily between individual, independent apps.
 *
 * See also section "Publish client Icon" on article @ref getting_started for
 * an important use case of this API part.
 *
 * @{
 */

/**
 * Prototype of a client supplied function that is called when any custom data
 * of foreign JACK clients appear, were replaced or removed. A client may
 * implement such a callback to efficiently react on clients publishing and
 * unpublishing any custom data.
 *
 * @param client_name - name of the foreign JACK client which modified the data
 * @param key - name of the new, replaced or removed custom data within the name
 *              space of the foreign client
 * @param change - 0 = removed (unpublished), 1 = added (published), 2 = replaced
 * @param arg - client specific structure, supplied when callback was registered
 */
typedef void (*JackCustomDataAppearanceCallback)(const char* client_name,
                                                 const char* key,
                                                 jack_custom_change_t change,
                                                 void* arg);

/**
 * Makes the passed data publicly available for every JACK client to be read.
 * The passed data will be copied into the JACK server process and kept in RAM
 * by the server for the life-time of the calling client. The data will be
 * available to be read by any JACK client under the given name (given by
 * @a key). Every JACK client has its own name space for custom data. Since the
 * data is copied by the server, the calling client may i.e. free the data from
 * its memory immediately after this call. As soon as the client was closed, all
 * custom data created by him will be unpublished automatically. In case some
 * custom data with name @a key already exists in the client's name space, it
 * will simply be replaced. It is also valid to pass a NULL pointer for @a data,
 * to publish some "empty" data. The latter might be used as kind of flag
 * variables.
 *
 * @param client - handle of the calling JACK client
 * @param key - some name under which the custom data shall be publicly visible
 * @param data - pointer to the data that shall be published
 *               (copied immediately, may be NULL)
 * @param size - size of the custom data in bytes
 * @return 0 on success
 */
int jack_custom_publish_data (jack_client_t* client, const char* key,
                              const void* data, size_t size) JACK_OPTIONAL_WEAK_EXPORT;

/**
 * Retrieve the custom data named by @a key from client given by @a client_name.
 * The data returned by this function is a copy, so it can be modified by the
 * caller of this function, without the original data to be modified in any way.
 * Since every JACK client has its own name space for custom data, the actual
 * unique ID of a specific custom data is identified by the combination of
 * @a client_name and @a key.
 *
 * The caller is responsible for calling jack_free() on any non-NULL returned
 * data at a later point.
 *
 * @param client - handle of the calling JACK client
 * @param client_name - client with which the custom data is associated with
 * @param key - name of the custom data
 * @param data - will be set to a copy of the custom data, will be set to NULL
 *               if sought custom data is empty or does not exist (you may pass
 *               a NULL pointer for just checking by return value whether the
 *               sought data exists at all)
 * @param size - optional pointer which will be set to the size of the custom
 *               data (in bytes), NULL can be passed if irrelevant
 * @return 0 on success, non-zero if client or data do not exist
 */
int jack_custom_get_data (jack_client_t* client, const char* client_name,
                          const char* key, void** data, size_t* size) JACK_OPTIONAL_WEAK_EXPORT;

/**
 * Allows to lookup all custom data currently published by the foreign JACK
 * client given by @a client_name. The caller will get a NULL-terminated list of
 * all data object names of the foreign JACK client reflected by @a client_name.
 * The caller may then pass one of the returned names to jack_custom_get_data()
 * to get the actual custom data.
 *
 * The caller is responsible for calling jack_free() on any non-NULL returned
 * result value !
 *
 * @param client - handle of the calling JACK client
 * @param client_name - name of the foreign JACK client whose custom data list
 *                     shall be retrieved
 * @return NULL-terminated name list
 */
const char** jack_custom_get_keys (jack_client_t* client,
                                   const char* client_name) JACK_OPTIONAL_WEAK_EXPORT;

/**
 * If a client wants some of its custom data to explicitly disappear during the
 * life-time of its JACK client, it may do so by calling this function.
 *
 * @param client - handle of the calling JACK client
 * @param key - name of the custom data to be removed
 * @return 0 on success
 */
int jack_custom_unpublish_data (jack_client_t* client, const char* key)
    JACK_OPTIONAL_WEAK_EXPORT;

/**
 * Tell the JACK server to call @a callback whenever new custom data appears
 * publicly or whenever some existing custom data was replaced or removed
 * (unpublished). The callback is only called for custom data
 * published by foreign JACK clients, not for any custom data published by the
 * calling client itself (given by @a client).
 *
 * All "notification events" are received in a seperated non RT thread, the
 * code in the supplied function does not need to be suitable for real-time
 * execution.
 *
 * NOTE: this function cannot be called while the client is activated
 * (after jack_activate has been called.)
 *
 * @param client - handle of calling JACK client
 * @param callback - client's callback implementation
 * @param arg - arbitrary client specific data that shall be passed on each
 *              callback call
 * @return 0 on success, otherwise a non-zero error code
 */
int jack_custom_set_data_appearance_callback (
        jack_client_t* client, JackCustomDataAppearanceCallback callback,
        void* arg
    ) JACK_OPTIONAL_WEAK_EXPORT;

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _JACK_CUSTOM_H */
